@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2017
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2019
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -408,6 +409,12 @@ public:
       return *this;
    }
 
+   template <typename Exp=Exponent, typename T>
+   RationalFunction<Coefficient,Exp> substitute_monomial(const T& exponent) const
+   {
+      return RationalFunction<Coefficient,Exp>(num.template substitute_monomial<Exp>(exponent),den.template substitute_monomial<Exp>(exponent));
+   }
+
    friend
    bool operator== (const RationalFunction& l, const RationalFunction& r)
    {
@@ -418,7 +425,7 @@ public:
    typename std::enable_if<fits_as_particle<T>::value, bool>::type
    operator== (const RationalFunction& l, const T& r)
    {
-      return l.den.unit() && l.num == r;
+      return l.den.is_one() && l.num == r;
    }
 
    template <typename T> friend
@@ -481,7 +488,7 @@ public:
 
    explicit operator const polynomial_type& () const
    {
-      if (!den.unit())
+      if (!den.is_one())
          throw std::runtime_error("Denominator is not one; cannot convert to a polynomial");
       return num;
    }
@@ -578,15 +585,25 @@ template <typename Coefficient, typename Exponent>
 struct spec_object_traits< Serialized< RationalFunction<Coefficient, Exponent> > > :
    spec_object_traits<is_composite> {
 
-   typedef RationalFunction<Coefficient, Exponent> masquerade_for;
+   using masquerade_for = RationalFunction<Coefficient, Exponent>;
 
-   typedef cons<typename RationalFunction<Coefficient, Exponent>::term_hash,
-                typename RationalFunction<Coefficient, Exponent>::term_hash> elements;
+   using terms_type = typename masquerade_for::term_hash;
+   using poly_type = typename masquerade_for::polynomial_type;
 
-   template <typename Me, typename Visitor>
-   static void visit_elements(Me& me, Visitor& v)
+   using elements = cons<terms_type,terms_type>;
+
+   template <typename Visitor>
+   static void visit_elements(const Serialized<masquerade_for>& me, Visitor& v)
    {
-      v << me.num.get_mutable_terms() << me.den.impl_ptr->get_mutable_terms();
+      v << me.num.get_terms() << me.den.get_terms();
+   }
+
+   template <typename Visitor>
+   static void visit_elements(Serialized<masquerade_for>& me, Visitor& v)
+   {
+      terms_type num, den;
+      v << num << den;
+      me = masquerade_for(poly_type(num,1),poly_type(den,1));
    }
 
 };
@@ -607,7 +624,7 @@ struct choose_generic_object_traits< RationalFunction<Coefficient, Exponent>, fa
    static
    bool is_one(const persistent_type& p)
    {
-      return numerator(p).unit() && denominator(p).unit();
+      return numerator(p).is_one() && denominator(p).is_one();
    }
 
    static
