@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2019
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -26,6 +27,7 @@
 #include "polymake/hash_map"
 #include "polymake/vector"
 #include "polymake/list"
+#include "polymake/optional"
 #include <algorithm>
 #include <sstream>
 
@@ -154,7 +156,7 @@ public:
    int size() const { return count_it(begin()); }
 };
 
-template <typename Permutation> inline
+template <typename Permutation>
 const PermutationCycles<Permutation>& permutation_cycles(const Permutation& p)
 {
    return reinterpret_cast<const PermutationCycles<Permutation>&>(p);
@@ -171,86 +173,87 @@ struct spec_object_traits< PermutationCycles<Permutation> >
    static const IO_separator_kind IO_separator=IO_sep_inherit;
 };
 
-template <typename Value, typename Comparator, typename enabled=void>
+template <typename Value, typename Comparator, typename ExpectDuplicates, typename enabled=void>
 struct permutation_map {
-   typedef Map<Value, int, Comparator> type;
+   using type = Map<Value, int, ComparatorTag<Comparator>, MultiTag<ExpectDuplicates>>;
 };
 
 template <typename Value>
-struct permutation_map<Value, operations::cmp, typename std::enable_if<!std::numeric_limits<Value>::is_specialized && is_ordered<Value>::value>::type> {
-   typedef Map<Value, int, operations::cmp> type;
+struct permutation_map<Value, operations::cmp, std::false_type, typename std::enable_if<!std::numeric_limits<Value>::is_specialized && is_ordered<Value>::value>::type> {
+   using type = Map<Value, int>;
 };
 
 template <typename Value>
-struct permutation_map<Value, operations::cmp, typename std::enable_if<std::numeric_limits<Value>::is_specialized>::type> {
-   typedef hash_map<Value, int> type;
+struct permutation_map<Value, operations::cmp, std::false_type, typename std::enable_if<std::numeric_limits<Value>::is_specialized>::type> {
+   using type = hash_map<Value, int>;
 };
 
-template <typename Input1, typename Input2, typename Output, typename Comparator>
-void find_permutation(Input1&& src1, Input2&& src2, Output&& dst, const Comparator& comparator)
+template <typename Value>
+struct permutation_map<Value, operations::cmp, std::true_type, typename std::enable_if<!std::numeric_limits<Value>::is_specialized && is_ordered<Value>::value>::type> {
+   using type = Map<Value, int, MultiTag<std::true_type>>;
+};
+
+template <typename Value>
+struct permutation_map<Value, operations::cmp, std::true_type, typename std::enable_if<std::numeric_limits<Value>::is_specialized>::type> {
+   using type = hash_map<Value, int, MultiTag<std::true_type>>;
+};
+
+template <typename Input1, typename Input2, typename Output, typename Comparator, typename ExpectDuplicates>
+bool find_permutation_impl(Input1&& src1, Input2&& src2, Output&& dst, const Comparator& comparator, ExpectDuplicates)
 {
-   typename permutation_map<typename iterator_traits<Input1>::value_type, Comparator>::type index_map;
+   typename permutation_map<typename iterator_traits<Input1>::value_type, Comparator, ExpectDuplicates>::type index_map;
 
-   for (int i=0; !src1.at_end(); ++src1, ++i)
-      index_map[*src1]=i;
+   for (int i = 0; !src1.at_end(); ++src1, ++i)
+      index_map.emplace(*src1, i);
 
    for (; !src2.at_end(); ++src2, ++dst) {
-      const auto where=index_map.find(*src2);
-      if (where == index_map.end()) {
-         std::string reason;
-         if (index_map.empty()) {
-            reason="not a permutation: first sequence is shorter";
-         } else {
-            std::ostringstream os;
-            wrap(os) << "not a permutation: no match for <" << *src2 << ">";
-            reason=os.str();
-         }
-         throw no_match(reason);
-      }
-      *dst=where->second;
+      const auto where = index_map.find(*src2);
+      if (where == index_map.end())
+         return false;
+      *dst = where->second;
       index_map.erase(where);
    }
 
-   if (!index_map.empty())
-      throw no_match("not a permutation: second sequence is shorter");
+   return index_map.empty();
 }
 
-template <typename Container1, typename Container2, typename Comparator> inline
-Array<int> find_permutation(const Container1& c1, const Container2& c2, const Comparator& comparator)
+template <typename Container1, typename Container2, typename Comparator = polymake::operations::cmp>
+optional<Array<int>>
+find_permutation(const Container1& c1, const Container2& c2, const Comparator& comparator = Comparator())
 {
    Array<int> perm(c1.size());
-   find_permutation(entire(c1), entire(c2), perm.begin(), comparator);
-   return perm;
+   if (find_permutation_impl(entire(c1), entire(c2), perm.begin(), comparator, std::false_type()))
+      return make_optional(std::move(perm));
+   else
+      return nullopt;
 }
 
-template <typename Container1, typename Container2> inline
-Array<int> find_permutation(const Container1& c1, const Container2& c2)
-{
-   return find_permutation(c1, c2, polymake::operations::cmp());
-}
-
-template <typename Container1, typename Container2, typename Comparator> inline
-bool are_permuted(const Container1& c1, const Container2& c2, const Comparator& comparator)
+template <typename Container1, typename Container2, typename Comparator = polymake::operations::cmp>
+optional<Array<int>>
+find_permutation_with_duplicates(const Container1& c1, const Container2& c2, const Comparator& comparator = Comparator())
 {
    Array<int> perm(c1.size());
-   try {
-      find_permutation(entire(c1), entire(c2), perm.begin(), comparator);
-      return true;
-   } catch (const no_match&) {
-      return false;
-   }
-   return false;
+   if (find_permutation_impl(entire(c1), entire(c2), perm.begin(), comparator, std::true_type()))
+      return make_optional(std::move(perm));
+   else
+      return nullopt;
 }
 
-template <typename Container1, typename Container2> inline
-bool are_permuted(const Container1& c1, const Container2& c2)
+template <typename Container1, typename Container2, typename Comparator = polymake::operations::cmp>
+bool are_permuted(const Container1& c1, const Container2& c2, const Comparator& comparator = Comparator())
 {
-   return are_permuted(c1,c2,polymake::operations::cmp());
+   return !!find_permutation(c1, c2, comparator);
 }
 
-template <typename Container, typename Permutation> inline
-typename std::enable_if<std::is_same<typename object_traits<Container>::generic_tag, is_container>::value,
-                        typename object_traits<Container>::persistent_type>::type
+template <typename Container1, typename Container2, typename Comparator=polymake::operations::cmp>
+bool are_permuted_with_duplicates(const Container1& c1, const Container2& c2, const Comparator& comparator = Comparator())
+{
+   return !!find_permutation_with_duplicates(c1, c2, comparator);
+}
+
+template <typename Container, typename Permutation>
+std::enable_if_t<std::is_same<typename object_traits<Container>::generic_tag, is_container>::value,
+                 typename object_traits<Container>::persistent_type>
 permuted(const Container& c, const Permutation& perm)
 {
    if (POLYMAKE_DEBUG) {
@@ -262,9 +265,9 @@ permuted(const Container& c, const Permutation& perm)
    return result;
 }
 
-template <typename Container, typename Permutation> inline
-typename std::enable_if<std::is_same<typename object_traits<Container>::generic_tag, is_container>::value,
-                        typename object_traits<Container>::persistent_type>::type
+template <typename Container, typename Permutation>
+std::enable_if_t<std::is_same<typename object_traits<Container>::generic_tag, is_container>::value,
+                 typename object_traits<Container>::persistent_type>
 permuted_inv(const Container& c, const Permutation& perm)
 {
    if (POLYMAKE_DEBUG) {
@@ -291,11 +294,11 @@ class permutation_iterator;
 
 class permutation_iterator_base {
 public:
-   typedef forward_iterator_tag iterator_category;
-   typedef std::vector<int> value_type;
-   typedef const value_type* pointer;
-   typedef const value_type& reference;
-   typedef ptrdiff_t difference_type;
+   using iterator_category = forward_iterator_tag;
+   using value_type = Array<int>;
+   using reference = value_type;
+   using pointer = const value_type*;
+   using difference_type = ptrdiff_t;
 protected:
    value_type perm;
 
@@ -314,8 +317,8 @@ class permutation_iterator<permutations_heap>
    : public permutation_iterator_base {
    friend class AllPermutations<permutations_heap>;
 public:
-   typedef permutation_iterator iterator;
-   typedef iterator const_iterator;
+   using iterator = permutation_iterator;
+   using const_iterator = iterator;
 
 protected:
    std::vector<int> cnt;
@@ -336,18 +339,18 @@ public:
       : permutation_iterator_base(n_arg)
       , cnt(size_t(n_arg), 0)
       , n(n_arg)
-      , pos(n_arg>1) {}
+      , pos(n_arg > 1) {}
 
    iterator& operator++ ()
    {
       do {
-         if (cnt[pos]<pos) {
+         if (cnt[pos] < pos) {
             std::swap(perm[pos], perm[pos%2 * cnt[pos]]);
             ++cnt[pos];
-            pos=1;
+            pos = 1;
             break;
          }
-         cnt[pos]=0;
+         cnt[pos] = 0;
       } while (++pos < n);
       return *this;
    }
@@ -357,7 +360,7 @@ public:
 
    bool operator== (const iterator& it) const
    {
-      return n==it.n && pos==it.pos && cnt==it.cnt;
+      return n == it.n && pos == it.pos && cnt == it.cnt;
    }
    bool operator!= (const iterator& it) const
    {
@@ -367,7 +370,7 @@ public:
    void rewind()
    {
       reset(n);
-      pos=1;
+      pos = 1;
       fill_range(entire(cnt), 0);
    }
 };
@@ -387,31 +390,31 @@ protected:
    void incr()
    {
       if (next==0) {
-         next=-1; return;
+         next = -1; return;
       }
-      int lim=n, offset=0;
-      std::vector<int>::iterator pos=move.begin();
+      int lim = n, offset = 0;
+      auto pos = move.begin();
       for (;;) {
-         int m=*pos;
-         if (m>0) {
-            if (m<lim) {
-               next=m+offset;
+         const int m = *pos;
+         if (m > 0) {
+            if (m < lim) {
+               next = m + offset;
                ++*pos;
                break;
             }
          } else {
-            if (m<-1) {
-               next=offset-m-1;
+            if (m < -1) {
+               next = offset - m - 1;
                ++*pos;
                break;
             }
             ++offset;
          }
          if (--lim > 1) {
-            *pos=-m;
+            *pos = -m;
             ++pos;
          } else {
-            next=0;
+            next = 0;
             break;
          }
       }
@@ -426,8 +429,8 @@ public:
       , n(n_arg)
       , next(n-1)
    {
-      if (next>0) {
-         next=1; move[0]=2;
+      if (next > 0) {
+         next = 1; move[0] = 2;
       }
    }
 
@@ -456,9 +459,9 @@ public:
    {
       reset(n);
       fill_range(entire(move), 1);
-      next=n-1;
-      if (next>1) {
-         next=1; move[0]=2;
+      next = n - 1;
+      if (next > 1) {
+         next = 1; move[0] = 2;
       }
    }
 };
@@ -557,19 +560,21 @@ AllPermutations<kind> all_permutations(int n)
 
 template <typename PermutationRef, typename Element=int>
 class PermutationMatrix
-   : public GenericMatrix< PermutationMatrix<PermutationRef,Element>, Element > {
+   : public GenericMatrix< PermutationMatrix<PermutationRef, Element>, Element > {
 protected:
-   alias<PermutationRef> perm;
+   using alias_t = alias<PermutationRef>;
+   alias_t perm;
    mutable std::vector<int> inv_perm;
 public:
-   typedef Element value_type;
-   typedef const Element& reference;
-   typedef reference const_reference;
+   using value_type = Element;
+   using reference = const Element&;
+   using const_reference = reference;
 
-   typedef typename alias<PermutationRef>::arg_type arg_type;
-   PermutationMatrix(arg_type perm_arg) : perm(perm_arg) {}
+   template <typename Arg, typename=std::enable_if_t<std::is_constructible<alias_t, Arg>::value>>
+   explicit PermutationMatrix(Arg&& perm_arg)
+      : perm(std::forward<Arg>(perm_arg)) {}
 
-   typename alias<PermutationRef>::const_reference get_perm() const { return *perm; }
+   decltype(auto) get_perm() const { return *perm; }
 
    const std::vector<int>& get_inv_perm() const
    {
@@ -602,21 +607,21 @@ public:
 template <typename PermutationRef, typename Element>
 class Rows< PermutationMatrix<PermutationRef, Element> >
    : public modified_container_pair_impl< Rows< PermutationMatrix<PermutationRef, Element> >,
-                                          mlist< Container1Tag< PermutationRef >,
-                                                 Container2Tag< constant_value_container<const Element&> >,
+                                          mlist< Container1RefTag< PermutationRef >,
+                                                 Container2RefTag< same_value_container<const Element&> >,
                                                  OperationTag< SameElementSparseVector_factory<2> >,
                                                  MasqueradedTop > > {
    typedef modified_container_pair_impl<Rows> base_t;
 protected:
    ~Rows();
 public:
-   const typename base_t::container1& get_container1() const
+   decltype(auto) get_container1() const
    {
       return this->hidden().get_perm();
    }
-   typename base_t::container2 get_container2() const
+   auto get_container2() const
    {
-      return one_value<Element>();
+      return same_value_container<const Element&>(one_value<Element>());
    }
    typename base_t::operation get_operation() const
    {
@@ -628,20 +633,20 @@ template <typename PermutationRef, typename Element>
 class Cols< PermutationMatrix<PermutationRef, Element> >
    : public modified_container_pair_impl< Cols< PermutationMatrix<PermutationRef, Element> >,
                                           mlist< Container1Tag< std::vector<int> >,
-                                                 Container2Tag< constant_value_container<const Element&> >,
+                                                 Container2RefTag< same_value_container<const Element&> >,
                                                  OperationTag< SameElementSparseVector_factory<2> >,
                                                  MasqueradedTop > > {
    typedef modified_container_pair_impl<Cols> base_t;
 protected:
    ~Cols();
 public:
-   const typename base_t::container1& get_container1() const
+   decltype(auto) get_container1() const
    {
       return this->hidden().get_inv_perm();
    }
-   typename base_t::container2 get_container2() const
+   auto get_container2() const
    {
-      return one_value<Element>();
+      return same_value_container<const Element&>(one_value<Element>());
    }
    typename base_t::operation get_operation() const
    {
@@ -649,28 +654,24 @@ public:
    }
 };
 
-template <typename Element, typename Permutation> inline
-PermutationMatrix<const Permutation&, Element>
-permutation_matrix(const Permutation& perm)
+template <typename Element, typename Permutation>
+auto permutation_matrix(Permutation&& perm)
 {
-   return perm;
+   return PermutationMatrix<Permutation, Element>(std::forward<Permutation>(perm));
 }
 
-template<typename Permutation> inline
-bool is_permutation(const Permutation& perm){
+template <typename Permutation>
+bool is_permutation(const Permutation& perm)
+{
    // check that perm really is a permutation
    Set<typename Permutation::value_type> values;
    
-   for (auto i=entire(perm);  !i.at_end();  ++i){
-      if(*i >= perm.size() or *i < 0){
-         return 0;
-      }
+   for (auto i=entire(perm); !i.at_end(); ++i) {
+      if (*i >= perm.size() or *i < 0)
+         return false;
       values.insert(*i);
    }
-   if(values.size() != perm.size() ){
-      return 0;
-   }
-   return 1;
+   return values.size() == perm.size();
 }
 
 template <typename Scalar>
@@ -720,7 +721,7 @@ permutation_subgroup_generators(const Array<Permutation>& gens,
       if (candidate_ok && candidate_gen != id)
          subgens += candidate_gen;
    }
-   return Array<Permutation>(subgens.size(), entire(subgens));
+   return Array<Permutation>(subgens.size(), subgens.begin());
 }
 
 } // end namespace pm
